<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
   xmlns:xs="http://www.w3.org/2001/XMLSchema" exclude-result-prefixes="xs" version="2.0"
   xpath-default-namespace="http://www.oxygenxml.com/ns/report">
   <xsl:template match="/">
      <html>
         <head>
            <title> Results </title>
            
            <style type="text/css">
               table{
                   color: #333;
                   font-family: verdana, arial, sans-serif;
                   border-collapse: collapse;
                   border-spacing: 0;
               }
               
               td,
               th{
                   border: 2px solid #CCC;
                   padding: 5px;
                   text-align: left;
                   vertical-align: middle;
               }
               
               th{
                   background-color: #BCBCBC;
                   font-weight: bold;
               }
               
               td{
                   background: #FAFAFA;
               }
            </style>
         </head>

         <xsl:variable name="incidentNodes" select="//incident"/>
         <xsl:variable name="hasSeverity" select="$incidentNodes/severity"/>
         <xsl:variable name="hasType" select="$incidentNodes/type"/>
         <xsl:variable name="hasLocation" select="$incidentNodes/location"/>
         <xsl:variable name="hasXPathLocation" select="$incidentNodes/xpath_location"/>

         <body>
            <table>
               <thead>
                  <tr>
                     <xsl:if test="$hasSeverity">
                        <th>Severity</th>
                     </xsl:if>
                     <xsl:if test="$hasType">
                        <th>Type</th>
                     </xsl:if>
                     <th>Description</th>
                     <xsl:if test="$hasXPathLocation">
                        <th>XPath Location</th>
                     </xsl:if>
                     <th>File</th>
                     <xsl:if test="$hasLocation">
                        <th>Location</th>
                     </xsl:if>
                  </tr>
               </thead>
               <tbody>
                  <xsl:for-each select="$incidentNodes">
                     <tr>
                        <xsl:if test="$hasSeverity">
                           <td>
                              <xsl:value-of select="severity"/>
                           </td>
                        </xsl:if>
                        <xsl:if test="$hasType">
                           <td>
                              <xsl:value-of select="type"/>
                           </td>
                        </xsl:if>
                        <td>
                           <xsl:value-of select="description"/>
                        </td>
                        <xsl:if test="$hasXPathLocation">
                           <td>
                              <xsl:value-of select="xpath_location"/>
                           </td>
                        </xsl:if>   
                        <td>
                           <xsl:value-of select="systemID"/>
                        </td>
                        <xsl:if test="$hasLocation">
                           <td>
                              <xsl:for-each select="location">
                                 <xsl:if test="start">
                                    <p>Start line <xsl:value-of select="start/line"/>:<xsl:value-of
                                          select="start/column"/>
                                    </p>
                                 </xsl:if>
                                 <xsl:if test="end">
                                    <p>End line <xsl:value-of select="start/line"/>:<xsl:value-of
                                          select="start/column"/>
                                    </p>
                                 </xsl:if>
                                 <xsl:if test="offset">
                                    <p>Offset: <xsl:value-of select="offset"/></p>
                                 </xsl:if>
                                 <xsl:if test="length">
                                    <p>Length: <xsl:value-of select="length"/></p>
                                 </xsl:if>
                              </xsl:for-each>
                           </td>
                        </xsl:if>
                     </tr>
                  </xsl:for-each>
               </tbody>
            </table>
         </body>
      </html>
   </xsl:template>
</xsl:stylesheet>
